<?php

namespace FiloBlu\StockaFix\Console\Command;

use FiloBlu\StockaFix\Helper\MailHelper;
use FiloBlu\StockaFix\Model\Report;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * Class ProcessCommand
 * @package FiloBlu\StockaFix\Console\Command
 */
class ProcessCommand extends Command
{
    /**
     * @var \FiloBlu\StockaFix\Model\ResourceModel\Report\CollectionFactory
     */
    protected $reportCollectionFactory;

    /**
     * @var \Magento\Framework\App\State
     */
    protected $state;

    /**
     * @var \Magento\Framework\DB\Adapter\AdapterInterface
     */
    protected $connection;

    /**
     * @var MailHelper
     */
    protected $mailHelper;

    /**
     * @var array
     */
    protected $results = [];

    /**
     * @var \Magento\Framework\Filesystem
     */
    protected $filesystem;

    /**
     * @var \Magento\Framework\App\Filesystem\DirectoryList
     */
    protected $directoryList;

    /**
     * @var \Magento\Framework\File\Csv
     */
    protected $csvProcessor;

    /**
     * @var InputInterface
     */
    protected $input;

    /**
     * @var OutputInterface
     */
    protected $output;

    /**
     * ProcessCommand constructor.
     * @param \FiloBlu\StockaFix\Model\ResourceModel\Report\CollectionFactory $reportCollectionFactory
     * @param \FiloBlu\StockaFix\Helper\MailHelper $mailHelper
     * @param \Magento\Framework\File\Csv $csvProcessor
     * @param \Magento\Framework\App\Filesystem\DirectoryList $directoryList
     * @param \Magento\Framework\Filesystem $filesystem
     * @param \Magento\Framework\App\State $state
     * @param string $name
     */
    public function __construct(
        \FiloBlu\StockaFix\Model\ResourceModel\Report\CollectionFactory $reportCollectionFactory,
        \FiloBlu\StockaFix\Helper\MailHelper $mailHelper,
        \Magento\Framework\File\Csv $csvProcessor,
        \Magento\Framework\App\Filesystem\DirectoryList $directoryList,
        \Magento\Framework\Filesystem $filesystem,
        \Magento\Framework\App\State $state,
        $name = null)
    {
        $this->reportCollectionFactory = $reportCollectionFactory;
        $this->mailHelper = $mailHelper;
        $this->state = $state;
        $this->filesystem = $filesystem;
        $this->directoryList = $directoryList;
        $this->csvProcessor = $csvProcessor;
        parent::__construct($name);
    }

    /**
     * {@inheritdoc}
     */
    protected function configure()
    {
        $this->setName('filoblu:stockafix:process')
            ->setDescription('Process active reports')
            ->setDefinition([]);

        parent::configure();
    }

    /**
     * {@inheritdoc}
     */
    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $this->output  = $output;
        $this->input = $input;

        try {
            $this->state->setAreaCode(\Magento\Framework\App\Area::AREA_ADMINHTML);
        } catch (\Exception $e) {
            $output->writeln('Areacode was already set, skipping!');
        }

        $collection = $this->reportCollectionFactory->create();
        $this->connection = $collection->getResource()->getConnection();

        $reports = $collection->addFieldToFilter('active', ['eq' => 1])->getItems();

        foreach ($reports as $report) {
            $output->write('<info>Processing ' . $report->getName() . ' ... </info>');

            if ($this->process($report)) {
                $output->writeln('<info>done</info>');

            } else {
                $output->writeln('<info>fail</info>');
            }
        }


    }

    /**
     * @param Report $report
     * @return bool
     */
    protected function process(Report $report)
    {
        try {

            $reportOutput = $this->connection->fetchAll($report->getSql(), [], \Zend_Db::FETCH_ASSOC);
            $csv = $this->writeToCsv($report, $reportOutput);
            $count = count($reportOutput);
            $this->results[] = [
                'report' => $report,
                'records' => $count,
                'csv' => $csv
            ];

            $to = explode(',', $report->getReceiverEmails());
            $this->mailHelper->sendReportMail($to, [$csv], ['report_name' => $report->getName(), 'report_count' => $count]);

            return true;
        } catch (\Exception $exception) {
            $this->output->writeln(PHP_EOL . '<error>'. $exception->getMessage() . '</error>');
            return false;
        }
    }

    /**
     * @param Report $report
     * @param array $data
     * @return string
     * @throws \Magento\Framework\Exception\FileSystemException
     */
    function writeToCsv(Report $report, $data = [])
    {
        $fileDirectoryPath = $this->directoryList->
            getPath(\Magento\Framework\App\Filesystem\DirectoryList::VAR_DIR) .
            DIRECTORY_SEPARATOR . 'stockafix' . DIRECTORY_SEPARATOR;

        if (!is_dir($fileDirectoryPath)) {
            mkdir($fileDirectoryPath, 0777, true);
        }

        $now = new \DateTime();
        $fileName = $report->getName() . '_' . $now->format("YmdHis") . '.csv';
        $filePath = $fileDirectoryPath . $fileName;

        $this->csvProcessor
            ->setEnclosure('"')
            ->setDelimiter(';')
            ->saveData($filePath, $data);

        return $filePath;
    }
}